/**********************************************************************
 * LOGGING & MONEY
 **********************************************************************/
function logAction(text) {
  const entry = document.createElement('div');
  entry.className = 'log-entry';
  entry.textContent = text;
  logContainer.appendChild(entry);
  logContainer.scrollTop = logContainer.scrollHeight;
}

function addMoney(amount, reason="") {
  money += amount;
  if(money < 0) money = 0;
  updateMoneyDisplay();
  if(reason) {
    logAction(`Money ${amount >= 0 ? '+' : ''}$${amount} from ${reason}. Total=$${money}`);
  }
}

function updateMoneyDisplay() {
  moneyDisplay.textContent = `Money: $${money}`;
  citizenCountDisplay.textContent = `Citizens: ${citizens.length}`;
}

/**********************************************************************
 * RANDOM UTILS
 **********************************************************************/
const firstNames = ["Al", "Bea", "Cal", "Dee", "Eve", "Fay", "Gil", "Hal", "Ian", "Joy",
                    "Kay", "Lee", "Max", "Ned", "Oda", "Pam", "Ray", "Sue", "Tim", "Ula",
                    "Vic", "Wyn", "Xan", "Yel", "Zed"];
const lastNames  = ["Apple", "Berry", "Cherry", "Delta", "Echo", "Flint", "Gran", "Hills",
                    "Iris", "Jones", "Knight", "Lemon", "Myer", "Noble", "Olson", "Prime",
                    "Quartz", "Row", "Smith", "Turn", "Umbra", "Vale", "Wick", "Xeno", "Yolk", "Zoom"];

function randomName() {
  const f = firstNames[Math.floor(Math.random() * firstNames.length)];
  const l = lastNames[Math.floor(Math.random() * lastNames.length)];
  return `${f} ${l}`;
}

function randInt(min, max) {
  return Math.floor(Math.random() * (max - min)) + min;
}

/**********************************************************************
 * MOVE & DISTANCE
 **********************************************************************/
// Default implementation of isWater - will be overridden by terrain.js
function isWater(x, y) {
  return false; // Default to no water
}

// Water movement penalty constant
const WATER_MOVEMENT_PENALTY = 0.5;

function moveToward(obj, tx, ty, speed=0.4) {
  let dx = tx - obj.x;
  let dy = ty - obj.y;
  let dist = Math.sqrt(dx*dx + dy*dy);
  
  // Apply water movement penalty if in water
  let actualSpeed = speed;
  if (isWater(obj.x, obj.y)) {
    actualSpeed *= WATER_MOVEMENT_PENALTY;
  }
  
  if(dist > 1) {
    obj.vx = (dx/dist) * actualSpeed;
    obj.vy = (dy/dist) * actualSpeed;
  } else {
    obj.vx = 0; 
    obj.vy = 0;
  }
}

function distance(x1, y1, x2, y2) {
  let dx = x2 - x1;
  let dy = y2 - y1;
  return Math.sqrt(dx*dx + dy*dy);
}

function knockback(ent, tx, ty, dist) {
  let dx = ent.x - tx;
  let dy = ent.y - ty;
  let length = Math.sqrt(dx*dx + dy*dy);
  if(length > 0.1) {
    ent.x += (dx/length) * dist;
    ent.y += (dy/length) * dist;
  }
}

/**********************************************************************
 * FINDERS
 **********************************************************************/
function findNearestResourceOfType(ref, rtype) {
  let best = null; 
  let bestD = Infinity;
  resources.forEach((res) => {
    if(res.type === rtype && res.amount > 0) {
      let d = distance(ref.x, ref.y, res.x, res.y);
      if(d < bestD) {
        bestD = d; 
        best = res;
      }
    }
  });
  return best;
}

function findNearestAnimalOfType(me, targetType) {
  let best = null;
  let bestD = Infinity;
  animals.forEach((a) => {
    if(a.type === targetType && !a.dead && a !== me) {
      let d = distance(me.x, me.y, a.x, a.y);
      if(d < bestD) {
        bestD = d;
        best = a;
      }
    }
  });
  return best;
}

function findHouseWithFruit() {
  return buildings.find(b => b.buildingType === "House" && b.completed && b.storedFruit > 0);
}

function findAnyCompletedHouse() {
  return buildings.find(b => b.buildingType === "House" && b.completed);
}

function findHouseNeedingFruit() {
  return buildings.find(b => b.buildingType === "House" && b.completed && b.storedFruit < b.maxFruit);
}

function findCompletedMarket() {
  return buildings.find(b => b.buildingType === "Market" && b.completed);
}

function findCompletedHospital() {
  return buildings.find(b => b.buildingType === "Hospital" && b.completed);
}

function findCompletedSchool() {
  return buildings.find(b => b.buildingType === "School" && b.completed);
}

/**********************************************************************
 * ROAD BUILDING WHEN HOUSE COMPLETES
 **********************************************************************/
function maybeBuildRoad(newHouse) {
  let otherHouses = buildings.filter(b => b.buildingType === "House" && b.completed && b !== newHouse);
  if(otherHouses.length === 0) return;
  
  let nearest = null; 
  let minD = Infinity;
  otherHouses.forEach((oh) => {
    let d = distance(newHouse.x, newHouse.y, oh.x, oh.y);
    if(d < minD) {
      minD = d;
      nearest = oh;
    }
  });
  
  if(!nearest) return;
  let road = createRoadSite(newHouse.x, newHouse.y, nearest.x, nearest.y);
  buildings.push(road);
  logAction(`A Road site created between House@(${Math.floor(newHouse.x)},${Math.floor(newHouse.y)}) & House@(${Math.floor(nearest.x)},${Math.floor(nearest.y)}).`);
}

/**********************************************************************
 * DEPOSIT WOOD LOGIC
 **********************************************************************/
function depositWoodToStorage(cit) {
  let buildingNeedingWood = buildings.find(b => !b.completed && b.deliveredWood < b.requiredWood);
  if(cit.carryingWood > 0 && !buildingNeedingWood) {
    // deposit wood to city center (0,0)
    let d = distance(cit.x, cit.y, 0, 0);
    if(d < 20) {
      cityStorage.wood += cit.carryingWood;
      logAction(`${cit.name} deposited ${cit.carryingWood} wood into city storage.`);
      cit.carryingWood = 0;
      return true;
    } else {
      moveToward(cit, 0, 0, 0.4);
      return true;
    }
  }
  return false;
}
